#ifdef __DPMI32__
#pragma inline
#endif

#ifdef	__DPMI16__
# include	<dos.h>
#	include	<mem.h>
#endif

#include	<process.h>
#include	<stdio.h>

#include	"dpmifunc.h"


BYTE2	allocRealMem( BYTE2 *selPtr, UINT nBytes )
{
	/* Allocate a block of nBytes from conventional memory.
		returns:
			real-mode segment of memory block allocated.
			If not enough memory, returns 0
			If other error, returns 0xFF
		selPtr:
			set to selector of memory block allocated.
		nBytes:
			The number of bytes to allocate.
	*/

  asm {
    mov ax, 100h
#ifdef	__DPMI32__
		mov ebx, nBytes		// Number of bytes.
		shr ebx, 4				// Number of paragraphs in EBX
#else
		mov bx, nBytes		// Number of bytes.
		shr bx, 4					// Number of paragraphs in BX
#endif
    int 31h

		jc	Error

#ifdef	__DPMI32__
		mov ebx, dword ptr selPtr
		mov word ptr [ebx], dx
#else
		mov bx, word ptr selPtr
		mov word ptr [bx], dx
#endif
		jmp	Exit
  }

Error:
	asm	{
		cmp	ax, 8					// Not enough memory
		jne unknownError
		mov ax, 0
		jmp	Exit
	}

unknownError:
	printf( "Untrapped memory error %d\n", _AX );
	exit( 0 );

Exit:
	return _AX;
}

void	freeRealMem( BYTE2 realSel )
{
  /* Free real memory block.
		realSel:
			Selector to block of conventional memory.
  */
  asm {
    mov ax, 101h
    mov dx, realSel
    int 31h
  }
}


void  protToRealnCopy( BYTE2 destSel, BYTE2 *srcOff, UINT nBytes )
{
	/*	Copy memory block of nBytes from srcOff to destSel.

			destSel:
				A selector to real-mode memory.
			srcOff:
				A selector to a protected-mode space buffer.  DS-based.
			nBytes:
				The number of bytes to copy.

			This routine assumes that destSel is a selector pointing to
			conventional memory.  Since the destSel may become an invalid
			selector later in the program (when the conventional memory is
			freed) the register which this routine uses to store the selector
			is saved and restored before returning.
	*/
  asm	push  es	// Save valid selector for es

#ifdef	__DPMI32__
  asm {
		/* ES:(E)DI <- destSel:0
		*/
    mov     di, destSel
    mov     es, di
    mov     edi, 0

		/* (E)SI <- srcOff, DS already holds selector.
		*/
    mov     esi, srcOff

		/* Setup (E)CX for an movsd (4-byte) copy.
		*/
    mov     ecx,nBytes
    mov     edx,ecx
    shr     ecx,2

    cld
    rep     movsd

		/* Now mov any bytes remaining.
		*/
    mov     ecx,edx
    and     ecx,3
    rep     movsb
  }
#else
	memcpy( MK_FP( destSel, 0 ), srcOff, nBytes );

#endif	def __DPMI32__ else

  asm	pop   es	// Restore valid selector
}


void	simulateRealModeInt( UCHAR theInt, FUNC300_STRUCT *parmsPtr )
{
  /* Simulate real mode interrupt.
			theInt:
				interrupt to generate
			parms_Ptr:
				pointer to a DPMI int 31, 300H parameters structure
	*/
  asm {
    mov ax, 300h
    mov bl, theInt
    mov bh, 0
    mov cx, 0
#ifdef	__DPMI32__
    mov edi, dword ptr parmsPtr
#else
		mov	di, word ptr parmsPtr
#endif
    int 31h
  }
}
